package org.seasar.s2hessian;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

import javax.servlet.http.HttpServletRequest;

import org.seasar.framework.beans.BeanDesc;
import org.seasar.framework.beans.factory.BeanDescFactory;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.servlet.S2ContainerServlet;

import com.caucho.hessian.io.HessianInput;
import com.caucho.hessian.io.HessianOutput;

/**
 * Proxy class for Hessian services.
 */
public class S2HessianSkeleton {
  
  /**
   * Create a new s2hessian skeleton.
   *
   */
  public S2HessianSkeleton()
  {

  }

  /**
   * Invoke the object with the request from the input stream.
   *
   * @param in the Hessian input stream
   * @param out the Hessian output stream
   */
  public void invoke(HessianInput in, HessianOutput out,HttpServletRequest req)
    throws Throwable
  {
    in.startCall();
    String methodName=in.getMethod();
	Object component = null;   
	
	String serviceName = req.getPathInfo();
	serviceName = serviceName.substring(1);
	S2Container container = S2ContainerServlet.getContainer();


		if (container.hasComponentDef(serviceName)) {
			component = container.getComponent(serviceName);
		} else {
			component = container.getComponent(Class.forName(serviceName));
		}

		BeanDesc beanDesc =
			BeanDescFactory.getBeanDesc(component.getClass());
	
	Method method=	beanDesc.getMethods(methodName)[0];
    Class []args = method.getParameterTypes();
    Object []values = new Object[args.length];

    for (int i = 0; i < args.length; i++)
      values[i] = in.readObject(args[i]);

    in.completeCall();

    Object result = null;

    try {

			result = beanDesc.invoke(component, methodName, values);
    } catch (Throwable e) {
      if (e instanceof InvocationTargetException)
        e = ((InvocationTargetException) e).getTargetException();
      out.startReply();
      out.writeFault("ServiceException", e.getMessage(), e);
      out.completeReply();
      return;
    }

    out.startReply();

    out.writeObject(result);
    
    out.completeReply();
  }
}
